/* ether.cpp -- Ethernet device functions

   This file is part of the SIB - simple IPX bridge

   Copyright (C) 2001 Szomraky Stefan

   The SIB is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of
   the License, or (at your option) any later version.

   The SIB is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with the SIB; see the file COPYING.
   If not, write to the Free Software Foundation, Inc.,
   59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
   
   Szomraky Stefan
   stsz@gmx.net
 */
#include "ether.h"

int getIfIndex( int fd, char *devname)
{
 struct ifreq ix;
 strcpy( ix.ifr_name, devname);
 
 ioctl( fd, SIOCGIFINDEX, &ix);
 
 return ix.ifr_ifindex;
}

int openDevice( char *devname)
{
 // for bind  
 struct sockaddr_ll dev;
 int fd;
 
 fd = socket(PF_PACKET, SOCK_RAW, htons(ETH_P_ALL));

 // fill in "ADDRESS"
 dev.sll_family = AF_PACKET;
 dev.sll_protocol = htons(ETH_P_ALL);
 dev.sll_ifindex = getIfIndex( fd, devname);
 
 if( dev.sll_ifindex <= 0) return -1;
 
 bind( fd, (struct sockaddr *) &dev, sizeof( dev));
 
 return fd;
}

int sendFrameETH( u_char *frame, u_int len)
{
 int rv;
 errno = 0;
 do {
  rv = send( config.fdd, frame, len, 0);
 } while( errno == EINTR);
 
 return rv; 
}

int getPromisc( void)
{
 struct ifreq ix;
 int fd = config.fdd;
 
 strcpy( ix.ifr_name, config.interfaceName);
 ioctl( fd, SIOCGIFFLAGS, &ix);
 
 return ix.ifr_flags & IFF_PROMISC;
}

int setPromisc( int state)
{
 struct ifreq ix;
 int fd = config.fdd;
 
 strcpy( ix.ifr_name, config.interfaceName);
 ioctl( fd, SIOCGIFFLAGS, &ix);

 if( state) 
  ix.ifr_flags |= IFF_PROMISC;
 else 
  ix.ifr_flags &= ~IFF_PROMISC;

 ioctl( fd, SIOCSIFFLAGS, &ix);
 return 0;
}