/* main.cpp -- main file

   This file is part of the SIB - simple IPX bridge

   Copyright (C) 2001 Szomraky Stefan

   The SIB is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of
   the License, or (at your option) any later version.

   The SIB is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with the SIB; see the file COPYING.
   If not, write to the Free Software Foundation, Inc.,
   59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
   
   Szomraky Stefan
   stsz@gmx.net
 */
#include "main.h"

Config config;
u_char *lzo_wrkmem;


int main( int argc, char *argv[])
{
 int maxfd; 
 long r;
 
 if( parseConfig( argc, argv) < 0) return 1;
 
 if( config.quiet) {
  int devnull;
  devnull = open( "/dev/null", O_WRONLY);
  dup2( devnull, 1);
  dup2( devnull, 2);
 }
 
 printGreet();
 
 if( config.debug) 
  printf( "debuglevel %u\n", (unsigned)(config.debug));
 
 if( config.statsInterval > 0) {
  progStartT = time( NULL);
  
  // only alarm works... Would somebody tell me why??
  // i want to use setitimer !!! 
  signal( SIGALRM, statsh);
  alarm( config.statsInterval);
  
  if( config.debug) printf( "%u seconds timer installed\n", config.statsInterval);
 }
 
 // for compression
#ifdef HAVE_LZO
 if( config.doCompression) {
  lzo_init();
  lzo_wrkmem = lzo_malloc(LZO1X_1_MEM_COMPRESS);
 }
#endif 

 u_char buf[MAX_BUF_LEN];
 fd_set watchset, inset;
  
 if( (config.fdd = openDevice( config.interfaceName)) <= 0) {
  printf( "could not find device %s\n", config.interfaceName);
  return 1;
 }
 
 if( config.doPromisc) {
  config.oldPromisc = getPromisc();
  setPromisc( 1);
  
  atexit( exith);
  signal( SIGINT, intrh);
  signal( SIGABRT, intrh);
  signal( SIGQUIT, intrh);
  signal( SIGHUP, SIG_IGN);
 }
 
 if( (config.fdu = openUDP( config.remoteHost, config.udpPort)) <= 0) return 1;
 
 fcntl( config.fdd, F_SETFL, O_NONBLOCK);
 fcntl( config.fdu, F_SETFL, O_NONBLOCK);
 
 FD_ZERO( &watchset);
 FD_SET( config.fdd, &watchset);
 FD_SET( config.fdu, &watchset);
 
 maxfd = config.fdd > config.fdu ? config.fdd : config.fdu;
 
 if( config.debug) printf( "DEV sock: %i, UDP sock: %i, maxfd: %i\n", config.fdu, config.fdd, maxfd);
   
 while( 1) {
  
  memcpy( &inset, &watchset, sizeof( inset));
  
  if( select( maxfd + 1, &inset, NULL, NULL, NULL) < 0) {
   // no REAL error, DUMB alarm interrupted select
   if( errno == EINTR) continue;
  
   perror( "select");
   return 1;
  }
  
  if( FD_ISSET( config.fdd, &inset)) {
    r = recv( config.fdd, buf, MAX_BUF_LEN, 0);
    if( config.debug >= 6) {
     printf( "read %li bytes from DEV socket (%i)\n", r, config.fdd);
     printf( "enMultiplexing (from: %s,", btoMAC( buf+6));
     printf(  "to: %s)\n", btoMAC( buf));
    }
    
    if( r > 0) enMultiplex( buf, r);
  } 
  
  if( FD_ISSET( config.fdu, &inset)) {
    r = recv( config.fdu, buf, MAX_BUF_LEN, 0);
    if( config.debug >= 6) printf( "read %li bytes from UDP socket (%i)\n", r, config.fdu);
    if( r > 0) deMultiplex( buf, r);
  }
 } 

 close( config.fdd);
 close( config.fdu); 
}



void statsh( int signum)
{
 printStats();
 alarm( config.statsInterval);
}

void exith( void)
{
 if( config.doPromisc)
  setPromisc( config.oldPromisc);
  
 close( config.fdd);
 close( config.fdu); 
 
 if( config.custlib) dlclose( config.custlib);
}

void intrh( int signum)
{
 exith(); 
 exit(1);
}
