/* network.cpp -- functions for INET access

   This file is part of the SIB - simple IPX bridge

   Copyright (C) 2001 Szomraky Stefan

   The SIB is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of
   the License, or (at your option) any later version.

   The SIB is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with the SIB; see the file COPYING.
   If not, write to the Free Software Foundation, Inc.,
   59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
   
   Szomraky Stefan
   stsz@gmx.net
 */
#include "network.h"

// opens the UDP socket, resolves the host ip, and saves it in
// a global variable (because we have to send that via SENDTO!!)
int openUDP( char *host, int port)
{
 int sock;
 // resolve IP
 hostent *hostent;
 in_addr inaddr;
 sockaddr_in address;
  
 if( inet_aton( host, &inaddr))
  hostent = gethostbyaddr((char *)&inaddr, sizeof( inaddr), AF_INET);
 else
  hostent = gethostbyname(host);
  
 if( !hostent) {
  perror( "Could not resolve host");
  return -1;
 } 

 printf( "opening UDP socket to %s on port %i...\n", hostent->h_name, port);
 
 // now we have the IP, create UDP socket
 if((sock = socket(PF_INET, SOCK_DGRAM, 0)) < 0) {
  perror( "Could not create socket");
  return -1;
 }
 
 address.sin_family = AF_INET;
 address.sin_port = htons(port);
 
 // bind the local port
 memset( &(address.sin_addr), 0, sizeof( address.sin_addr));
 bind( sock, (sockaddr *) &address, sizeof( address));
 
 // connect to remote
 memcpy( &(address.sin_addr), hostent->h_addr_list[0], sizeof( address.sin_addr));
 connect( sock, (sockaddr *)&address, sizeof( address));

 // set MTU autodiscovery
 int d = IP_PMTUDISC_DO;
 setsockopt( sock, SOL_IP, IP_MTU_DISCOVER, &d, sizeof(d));
  
 return sock;
}

int sendFrameUDP( u_char *frame, u_int len)
{
 int rv;
 if( len > config.MTU) return sendFrameUDP_MTU( frame, len);

 errno = 0;
 do {
   rv = send( config.fdu, frame, len, 0);
   if( errno == EMSGSIZE) return sendFrameUDP_MTU( frame, len);
 } while( errno == EINTR);
 
 return rv;
}

int sendFrameUDP_MTU( u_char *frame, u_int len)
{
 u_int dummy, oldMTU = config.MTU;
 
 config.MTUseq++;
 
 if( config.debug >= 3) printf( "sendFrameUPD_MTU called with size %u, MTU: %u\n", len, config.MTU);


 if( getsockopt( config.fdu, SOL_IP, IP_MTU, &(config.MTU), &dummy))
  return -1;


 
 // UDP + IP header len
 if( config.MTU <= 28) return -1;
 config.MTU -= 28;

 
 if( config.debug && oldMTU != config.MTU) 
   printf( "new MTU: %u\n", config.MTU);

 
 static u_char block[MAX_BUF_LEN];
 u_short mlen=0;
 u_char i=0;
 u_char parts = len / config.MTU;
 if( parts * config.MTU != len ) parts++;
 
 do {
  memset( block, 0, MAX_BUF_LEN);
  block[0] = MSG_SPLIT;
 
  mlen = (len > config.MTU-4 ? config.MTU -4 : len);
  block[1] = config.MTUseq;	// MP sequence
  block[2] = i;			// current chunk
  block[3] = parts;		// # of chunks
  memcpy( block+4, frame, mlen);
 
  if( config.debug >= 2) 
   printf( "sending multipacket part %u, with %u bytes (%u last), MTUseq: %u\n", (unsigned)(i+1), mlen, len-mlen, config.MTUseq);

  errno = 0;
  // TODO: handle MTU change while sending multipacket
  if( send( config.fdu, block, mlen+4, 0) < 0) {
   if( config.debug >= 2) {
    printf( "failed\n");
    perror( "send MTU");
   }    
   return -1;
  }
    
  frame += mlen;
  len -= mlen;
  i++;
 } while( len);
 
 return 0;
}
