/* util.cpp -- some parsing functions

   This file is part of the SIB - simple IPX bridge

   Copyright (C) 2001 Szomraky Stefan

   The SIB is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of
   the License, or (at your option) any later version.

   The SIB is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with the SIB; see the file COPYING.
   If not, write to the Free Software Foundation, Inc.,
   59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
   
   Szomraky Stefan
   stsz@gmx.net
 */
#include "util.h"

u_char MACLIST[MAX_MLL][6];
u_int MLC = 0;
int MLI = -1;

// adds a MAC to the MAC list
bool addLocal( u_char *mac)
{
 if( isLocal( mac)) return true;

 if( !MLC) 
  memset( MACLIST, 0, 6*MAX_MLL);
 
 MLC++; MLI++;
 if( MLC >= MAX_MLL) { 
  MLC = MAX_MLL;
  MLI = 0;
 } 
 
 memcpy( MACLIST[MLI], mac, 6);
 
 return false;
}

// returns true if a MAC is in the local MAC list
bool isLocal( u_char *mac)
{
 if( !MLC)
  return false;
 
 for( u_int i = 0; i < MLC; i++)
  if( !memcmp( MACLIST[i], mac, 6)) return true; 
 
 return false; 
}

// returns true if a frame has to be filtered
bool filterFrame( u_char *frame, u_int len)
{
 if( frame[12] == 8) // IP 
  return config.protoMask.IP ? false : true;
 
 if( frame[12] < 6)  // IPX
  return config.protoMask.IPX ? false : true;
  
 return true;
}

// checks if a frame is SMB (and others) over IPX instead of standard IPX
bool isIPXSMB( u_char *frame, u_int len) 
{
 if( frame[12] < 6 && frame[22] == 0x14) return true;

 return false;
}

char *btoMAC( u_char *mac)
{
 static char buf[18];
 
 memset( buf, 0, 18);
 
 for( int i = 0; i < 6; i++)
  sprintf( buf, "%s%02hX%s", buf, (u_short)mac[i], i < 5 ? ":" : "");
  
 return buf; 
}

int atoProtoIndex( char *proto)
{
 if( !strcasecmp( proto, "ALL"))
  return PROTO_ALL;
  
 if( !strcasecmp( proto, "IP"))
  return PROTO_IP;
 
 if( !strcasecmp( proto, "IPX"))
  return PROTO_IPX;
 
 return -1;
}
